/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <zephyr/bluetooth/bluetooth.h>
#include <bluetooth/mesh/models.h>
#include <dk_buttons_and_leds.h>
#include "model_handler.h"


static void red_sensor_cli_data_cb(struct bt_mesh_sensor_cli *cli,
			       struct bt_mesh_msg_ctx *ctx,
			       const struct bt_mesh_sensor_type *sensor,
			       const struct sensor_value *value)
{
	if (sensor->id == bt_mesh_sensor_present_amb_light_level.id) {
		printk("Red light level: %s\n",
				bt_mesh_sensor_ch_str(value));
	}
}

static void green_sensor_cli_data_cb(struct bt_mesh_sensor_cli *cli,
			       struct bt_mesh_msg_ctx *ctx,
			       const struct bt_mesh_sensor_type *sensor,
			       const struct sensor_value *value)
{
	if (sensor->id == bt_mesh_sensor_present_amb_light_level.id) {
		printk("Green light level: %s\n",
				bt_mesh_sensor_ch_str(value));
	}
}

static void blue_sensor_cli_data_cb(struct bt_mesh_sensor_cli *cli,
			       struct bt_mesh_msg_ctx *ctx,
			       const struct bt_mesh_sensor_type *sensor,
			       const struct sensor_value *value)
{
	if (sensor->id == bt_mesh_sensor_present_amb_light_level.id) {
		printk("Blue light level: %s\n",
				bt_mesh_sensor_ch_str(value));
	}
}

static const struct bt_mesh_sensor_cli_handlers bt_mesh_red_sensor_cli_handlers = {
	.data = red_sensor_cli_data_cb,
};

static const struct bt_mesh_sensor_cli_handlers bt_mesh_green_sensor_cli_handlers = {
	.data = green_sensor_cli_data_cb,
};

static const struct bt_mesh_sensor_cli_handlers bt_mesh_blue_sensor_cli_handlers = {
	.data = blue_sensor_cli_data_cb,
};

static struct bt_mesh_sensor_cli red_sensor_cli =
	BT_MESH_SENSOR_CLI_INIT(&bt_mesh_red_sensor_cli_handlers);

static struct bt_mesh_sensor_cli green_sensor_cli =
	BT_MESH_SENSOR_CLI_INIT(&bt_mesh_green_sensor_cli_handlers);

static struct bt_mesh_sensor_cli blue_sensor_cli =
	BT_MESH_SENSOR_CLI_INIT(&bt_mesh_blue_sensor_cli_handlers);

/* Set up a repeating delayed work to blink the DK's LEDs when attention is
 * requested.
 */
static struct k_work_delayable attention_blink_work;
static bool attention;

static void attention_blink(struct k_work *work)
{
	static int idx;
	const uint8_t pattern[] = {
		BIT(0) | BIT(1),
		BIT(1) | BIT(2),
		BIT(2) | BIT(3),
		BIT(3) | BIT(0),
	};

	if (attention) {
		dk_set_leds(pattern[idx++ % ARRAY_SIZE(pattern)]);
		k_work_reschedule(&attention_blink_work, K_MSEC(30));
	} else {
		dk_set_leds(DK_NO_LEDS_MSK);
	}
}

static void attention_on(struct bt_mesh_model *mod)
{
	attention = true;
	k_work_reschedule(&attention_blink_work, K_NO_WAIT);
}

static void attention_off(struct bt_mesh_model *mod)
{
	/* Will stop rescheduling blink timer */
	attention = false;
}

static const struct bt_mesh_health_srv_cb health_srv_cb = {
	.attn_on = attention_on,
	.attn_off = attention_off,
};

static struct bt_mesh_health_srv health_srv = {
	.cb = &health_srv_cb,
};

BT_MESH_HEALTH_PUB_DEFINE(health_pub, 0);

static struct bt_mesh_elem elements[] = {
	BT_MESH_ELEM(1,
		     BT_MESH_MODEL_LIST(BT_MESH_MODEL_CFG_SRV,
					BT_MESH_MODEL_HEALTH_SRV(&health_srv,
								 &health_pub),
					BT_MESH_MODEL_SENSOR_CLI(&red_sensor_cli)),
		     BT_MESH_MODEL_NONE),
	BT_MESH_ELEM(2,
		     BT_MESH_MODEL_LIST(
					BT_MESH_MODEL_SENSOR_CLI(&green_sensor_cli)),
		     BT_MESH_MODEL_NONE),
	BT_MESH_ELEM(3,
		     BT_MESH_MODEL_LIST(
					BT_MESH_MODEL_SENSOR_CLI(&blue_sensor_cli)),
		     BT_MESH_MODEL_NONE),
};

static const struct bt_mesh_comp comp = {
	.cid = CONFIG_BT_COMPANY_ID,
	.elem = elements,
	.elem_count = ARRAY_SIZE(elements),
};

const struct bt_mesh_comp *model_handler_init(void)
{
	k_work_init_delayable(&attention_blink_work, attention_blink);

	return &comp;
}
