/*
 * Copyright (c) 2020 Nordic Semiconductor ASA
 *
 * SPDX-License-Identifier: LicenseRef-Nordic-5-Clause
 */

#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/bluetooth/bluetooth.h>
#include <bluetooth/mesh/models.h>
#include <dk_buttons_and_leds.h>
#include "model_handler.h"

const struct device *dev = DEVICE_DT_GET_ONE(rohm_bh1749);

static int red_light_level_get(struct bt_mesh_sensor_srv *srv,
			 struct bt_mesh_sensor *sensor,
			 struct bt_mesh_msg_ctx *ctx,
			 struct sensor_value *rsp)
{
	int err;

	sensor_sample_fetch(dev);
	err = sensor_channel_get(dev, SENSOR_CHAN_RED, rsp);
	if (err) {
		printk("Error getting red channel (%d)\n", err);
	}
	return err;
}

static int green_light_level_get(struct bt_mesh_sensor_srv *srv,
			 struct bt_mesh_sensor *sensor,
			 struct bt_mesh_msg_ctx *ctx,
			 struct sensor_value *rsp)
{
	int err;

	sensor_sample_fetch(dev);
	err = sensor_channel_get(dev, SENSOR_CHAN_GREEN, rsp);
	if (err) {
		printk("Error getting green channel (%d)\n", err);
	}
	return err;
}

static int blue_light_level_get(struct bt_mesh_sensor_srv *srv,
			 struct bt_mesh_sensor *sensor,
			 struct bt_mesh_msg_ctx *ctx,
			 struct sensor_value *rsp)
{
	int err;

	sensor_sample_fetch(dev);
	err = sensor_channel_get(dev, SENSOR_CHAN_BLUE, rsp);
	if (err) {
		printk("Error getting blue channel (%d)\n", err);
	}
	return err;
}

static struct bt_mesh_sensor red_light_level = {
	.type = &bt_mesh_sensor_present_amb_light_level,
	.get = red_light_level_get,
};

static struct bt_mesh_sensor green_light_level = {
	.type = &bt_mesh_sensor_present_amb_light_level,
	.get = green_light_level_get,
};

static struct bt_mesh_sensor blue_light_level = {
	.type = &bt_mesh_sensor_present_amb_light_level,
	.get = blue_light_level_get,
};

static struct bt_mesh_sensor *const red_sensors[] = {
	&red_light_level,
};

static struct bt_mesh_sensor *const green_sensors[] = {
	&green_light_level,
};

static struct bt_mesh_sensor *const blue_sensors[] = {
	&blue_light_level,
};

static struct bt_mesh_sensor_srv red_sensor_srv =
	BT_MESH_SENSOR_SRV_INIT(red_sensors, ARRAY_SIZE(red_sensors));

static struct bt_mesh_sensor_srv green_sensor_srv =
	BT_MESH_SENSOR_SRV_INIT(green_sensors, ARRAY_SIZE(green_sensors));

static struct bt_mesh_sensor_srv blue_sensor_srv =
	BT_MESH_SENSOR_SRV_INIT(blue_sensors, ARRAY_SIZE(blue_sensors));

/* Set up a repeating delayed work to blink the DK's LEDs when attention is
 * requested.
 */
static struct k_work_delayable attention_blink_work;
static bool attention;

static void attention_blink(struct k_work *work)
{
	static int idx;
	const uint8_t pattern[] = {
#if DT_NODE_EXISTS(DT_ALIAS(led0))
		BIT(0),
#endif
#if DT_NODE_EXISTS(DT_ALIAS(led1))
		BIT(1),
#endif
#if DT_NODE_EXISTS(DT_ALIAS(led2))
		BIT(2),
#endif
#if DT_NODE_EXISTS(DT_ALIAS(led3))
		BIT(3),
#endif
	};

	if (attention) {
		dk_set_leds(pattern[idx++ % ARRAY_SIZE(pattern)]);
		k_work_reschedule(&attention_blink_work, K_MSEC(30));
	} else {
		dk_set_leds(DK_NO_LEDS_MSK);
	}
}

static void attention_on(struct bt_mesh_model *mod)
{
	attention = true;
	k_work_reschedule(&attention_blink_work, K_NO_WAIT);
}

static void attention_off(struct bt_mesh_model *mod)
{
	/* Will stop rescheduling blink timer */
	attention = false;
}

static const struct bt_mesh_health_srv_cb health_srv_cb = {
	.attn_on = attention_on,
	.attn_off = attention_off,
};

static struct bt_mesh_health_srv health_srv = {
	.cb = &health_srv_cb,
};

BT_MESH_HEALTH_PUB_DEFINE(health_pub, 0);

static struct bt_mesh_elem elements[] = {
	BT_MESH_ELEM(1,
		     BT_MESH_MODEL_LIST(BT_MESH_MODEL_CFG_SRV,
					BT_MESH_MODEL_HEALTH_SRV(&health_srv,
								 &health_pub),
					BT_MESH_MODEL_SENSOR_SRV(&red_sensor_srv)),
		     BT_MESH_MODEL_NONE),
	BT_MESH_ELEM(2,
		     BT_MESH_MODEL_LIST(
					BT_MESH_MODEL_SENSOR_SRV(&green_sensor_srv)),
		     BT_MESH_MODEL_NONE),
	BT_MESH_ELEM(3,
		     BT_MESH_MODEL_LIST(
					BT_MESH_MODEL_SENSOR_SRV(&blue_sensor_srv)),
		     BT_MESH_MODEL_NONE),
};

static const struct bt_mesh_comp comp = {
	.cid = CONFIG_BT_COMPANY_ID,
	.elem = elements,
	.elem_count = ARRAY_SIZE(elements),
};

const struct bt_mesh_comp *model_handler_init(void)
{
	k_work_init_delayable(&attention_blink_work, attention_blink);

	if (!device_is_ready(dev)) {
		printk("Light sensor not ready\n");
	} else {
		printk("Light sensor (%s) initiated\n", dev->name);
	}

	return &comp;
}
